@@ -1,4 +1,3 @@
-/* vi: set sw=4 ts=4: */
 /*
  * This file contains the helper routines to load an ELF shared
  * library into memory and add the symbol table info to the chain.
@@ -132,7 +131,7 @@ _dl_protect_relro (struct elf_resolve *l)
 /* This function's behavior must exactly match that
  * in uClibc/ldso/util/ldd.c */
 static struct elf_resolve *
-search_for_named_library(const char *name, unsigned rflags, const char *path_list,
+search_for_named_library(const char *name, unsigned int rflags, const char *path_list,
 	struct dyn_elf **rpnt, const char* origin)
 {
 	char *mylibname;
@@ -162,7 +161,9 @@ search_for_named_library(const char *name, unsigned rflags, const char *path_lis
 
 		if (plen >= 7 && _dl_memcmp(p, "$ORIGIN", 7) == 0) {
 			int olen;
-			if (rflags && plen != 7)
+			/* $ORIGIN is not expanded for SUID/GUID programs
+			   (except if it is $ORIGIN alone) */
+			if ((rflags & __RTLD_SECURE) && plen != 7)
 				continue;
 			if (origin == NULL)
 				continue;
@@ -194,7 +195,7 @@ search_for_named_library(const char *name, unsigned rflags, const char *path_lis
 unsigned long _dl_error_number;
 unsigned long _dl_internal_error_number;
 
-struct elf_resolve *_dl_load_shared_library(unsigned rflags, struct dyn_elf **rpnt,
+struct elf_resolve *_dl_load_shared_library(unsigned int rflags, struct dyn_elf **rpnt,
 	struct elf_resolve *tpnt, char *full_libname, int attribute_unused trace_loaded_objects)
 {
 	char *pnt;
@@ -305,6 +306,19 @@ struct elf_resolve *_dl_load_shared_library(unsigned rflags, struct dyn_elf **rp
 		}
 	}
 #endif
+
+#ifdef LDSO_MULTILIB_DIR
+	/* If multilib directory is selected, search it before falling back to
+	   standard lib directories. */
+	_dl_if_debug_dprint("\tsearching multilib lib path list\n");
+	tpnt1 = search_for_named_library(libname, rflags,
+					UCLIBC_RUNTIME_PREFIX LDSO_MULTILIB_DIR ":"
+					UCLIBC_RUNTIME_PREFIX "usr/" LDSO_MULTILIB_DIR,
+					rpnt, NULL);
+	if (tpnt1 != NULL)
+		return tpnt1;
+#endif
+
 #if defined SHARED && defined __LDSO_SEARCH_INTERP_PATH__
 	/* Look for libraries wherever the shared library loader
 	 * was installed */
@@ -313,6 +327,7 @@ struct elf_resolve *_dl_load_shared_library(unsigned rflags, struct dyn_elf **rp
 	if (tpnt1 != NULL)
 		return tpnt1;
 #endif
+
 	/* Lastly, search the standard list of paths for the library.
 	   This list must exactly match the list in uClibc/ldso/util/ldd.c */
 	_dl_if_debug_dprint("\tsearching full lib path list\n");
@@ -493,7 +508,7 @@ map_writeable (int infile, ElfW(Phdr) *ppnt, int piclib, int flags,
  * are required.
  */
 
-struct elf_resolve *_dl_load_elf_shared_library(unsigned rflags,
+struct elf_resolve *_dl_load_elf_shared_library(unsigned int rflags,
 	struct dyn_elf **rpnt, const char *libname)
 {
 	ElfW(Ehdr) *epnt;
@@ -532,7 +547,7 @@ struct elf_resolve *_dl_load_elf_shared_library(unsigned rflags,
 	}
 	/* If we are in secure mode (i.e. a setuid/gid binary using LD_PRELOAD),
 	   we don't load the library if it isn't setuid. */
-	if (rflags & DL_RESOLVE_SECURE) {
+	if (rflags & __RTLD_SECURE) {
 		if (!(st.st_mode & S_ISUID)) {
 			_dl_close(infile);
 			return NULL;
@@ -544,11 +559,12 @@ struct elf_resolve *_dl_load_elf_shared_library(unsigned rflags,
 		if (tpnt->st_dev == st.st_dev && tpnt->st_ino == st.st_ino) {
 			/* Already loaded */
 			tpnt->usage_count++;
+			tpnt->init_flag |= DL_OPENED2;
 			_dl_close(infile);
 			return tpnt;
 		}
 	}
-	if (rflags & DL_RESOLVE_NOLOAD) {
+	if (rflags & RTLD_NOLOAD) {
 		_dl_close(infile);
 		return NULL;
 	}
@@ -812,7 +828,7 @@ struct elf_resolve *_dl_load_elf_shared_library(unsigned rflags,
 		DL_RELOC_ADDR(DL_GET_RUN_ADDR(tpnt->loadaddr, tpnt->mapaddr),
 		epnt->e_phoff);
 	tpnt->n_phent = epnt->e_phnum;
-	tpnt->rtld_flags |= rtld_flags;
+	tpnt->rtld_flags = rflags | rtld_flags;
 #ifdef __LDSO_STANDALONE_SUPPORT__
 	tpnt->l_entry = epnt->e_entry;
 #endif
@@ -881,6 +897,8 @@ struct elf_resolve *_dl_load_elf_shared_library(unsigned rflags,
 #endif
 	(*rpnt)->dyn = tpnt;
 	tpnt->usage_count++;
+	if (tpnt->rtld_flags & RTLD_NODELETE)
+		tpnt->usage_count++;
 #ifdef __LDSO_STANDALONE_SUPPORT__
 	tpnt->libtype = (epnt->e_type == ET_DYN) ? elf_lib : elf_executable;
 #else
