@@ -1,4 +1,3 @@
-/* vi: set sw=4 ts=4: */
 /*
  * fstat() for uClibc
  *
@@ -21,7 +20,7 @@ int fstat(int fd, struct stat *buf)
 }
 libc_hidden_def(fstat)
 
-#elif __WORDSIZE == 64 && defined __NR_newfstatat
+#elif __WORDSIZE == 64 && defined __NR_newfstatat && __aarch64__
 #include <fcntl.h>
 
 int fstat(int fd, struct stat *buf)
